package com.scottmcfurry.psx.input;

import com.androidemu.Emulator;

import android.content.Context;
import android.content.res.Configuration;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

public class SensorKeypad implements SensorEventListener {
	private static final float THRESHOLD = 3.0f;//

	private Context context;//
	private GameKeyListener gameKeyListener;//
	private int keyStates;//

	public SensorKeypad(Context ctx) {
		context = ctx;
	}

	public final int getKeyStates() {
		return keyStates;
	}

	public void setGameKeyListener(GameKeyListener l) {
		if (gameKeyListener == l)
			return;

		SensorManager sensorManager = (SensorManager)
				context.getSystemService(Context.SENSOR_SERVICE);

		if (gameKeyListener != null)
			sensorManager.unregisterListener(this);

		gameKeyListener = l;
		if (gameKeyListener != null) {
			Sensor sensor = sensorManager.getDefaultSensor(
					Sensor.TYPE_ORIENTATION);
			sensorManager.registerListener(this,
					sensor, SensorManager.SENSOR_DELAY_GAME);
		}
	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}

	public void onSensorChanged(SensorEvent event) {
		float leftRight, upDown;

		Configuration config = context.getResources().getConfiguration();
		if (config.orientation == Configuration.ORIENTATION_LANDSCAPE) {
			leftRight = -event.values[1];
			upDown = event.values[2];
		} else {
			leftRight = -event.values[2];
			upDown = -event.values[1];
		}

		int states = 0;
		if (leftRight < -THRESHOLD/2)
			states |= Emulator.GAMEPAD_LEFT;
		else if (leftRight > THRESHOLD)
			states |= Emulator.GAMEPAD_RIGHT;

		if (states != keyStates) {
			keyStates = states;
			if (gameKeyListener != null)
				gameKeyListener.onGameKeyChanged();
		}
	}
}
